@@ -10,7 +10,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "lib.h"
@@ -29,9 +29,9 @@ char *pv_fmt_dup(const struct physical_volume *pv)
 	return dm_pool_strdup(pv->vg->vgmem, pv->fmt->name);
 }
 
-char *pv_name_dup(const struct physical_volume *pv)
+char *pv_name_dup(struct dm_pool *mem, const struct physical_volume *pv)
 {
-	return dm_pool_strdup(pv->vg->vgmem, dev_name(pv->dev));
+	return dm_pool_strdup(mem ? mem : pv->vg->vgmem, dev_name(pv->dev));
 }
 
 /*
@@ -42,9 +42,9 @@ struct id pv_id(const struct physical_volume *pv)
 	return pv_field(pv, id);
 }
 
-char *pv_uuid_dup(const struct physical_volume *pv)
+char *pv_uuid_dup(struct dm_pool *mem, const struct physical_volume *pv)
 {
-	return id_format_and_copy(pv->vg->vgmem, &pv->id);
+	return id_format_and_copy(mem ? mem : pv->vg->vgmem, &pv->id);
 }
 
 char *pv_tags_dup(const struct physical_volume *pv)
@@ -69,7 +69,8 @@ struct device *pv_dev(const struct physical_volume *pv)
 
 const char *pv_vg_name(const struct physical_volume *pv)
 {
-	return pv_field(pv, vg_name);
+	/* Avoid exposing internal orphan names to users */
+	return (!is_orphan(pv)) ? pv_field(pv, vg_name) : "";
 }
 
 const char *pv_dev_name(const struct physical_volume *pv)
@@ -88,6 +89,7 @@ uint64_t pv_dev_size(const struct physical_volume *pv)
 
 	if (!dev_get_size(pv->dev, &size))
 		size = 0;
+
 	return size;
 }
 
@@ -99,6 +101,7 @@ uint64_t pv_size_field(const struct physical_volume *pv)
 		size = pv->size;
 	else
 		size = (uint64_t) pv->pe_count * pv->pe_size;
+
 	return size;
 }
 
@@ -111,6 +114,7 @@ uint64_t pv_free(const struct physical_volume *pv)
 	else
 		freespace = (uint64_t)
 			(pv->pe_count - pv->pe_alloc_count) * pv->pe_size;
+
 	return freespace;
 }
 
@@ -154,14 +158,17 @@ uint32_t pv_mda_count(const struct physical_volume *pv)
 	struct lvmcache_info *info;
 
 	info = lvmcache_info_from_pvid((const char *)&pv->id.uuid, 0);
+
 	return info ? lvmcache_mda_count(info) : UINT64_C(0);
 }
 
 static int _count_unignored(struct metadata_area *mda, void *baton)
 {
 	uint32_t *count = baton;
+
 	if (!mda_is_ignored(mda))
 		(*count) ++;
+
 	return 1;
 }
 
@@ -174,6 +181,7 @@ uint32_t pv_mda_used_count(const struct physical_volume *pv)
 	if (!info)
 		return 0;
 	lvmcache_foreach_mda(info, _count_unignored, &used_count);
+
 	return used_count;
 }
 
@@ -200,16 +208,53 @@ int is_missing_pv(const struct physical_volume *pv)
 	return pv_field(pv, status) & MISSING_PV ? 1 : 0;
 }
 
+int is_used_pv(const struct physical_volume *pv)
+{
+	struct lvmcache_info *info;
+	uint32_t ext_flags;
+
+	if (!pv->fmt)
+		return 0;
+
+	if (!is_orphan(pv))
+		return 1;
+
+	if (!(pv->fmt->features & FMT_PV_FLAGS))
+		return 0;
+
+	if (!(info = lvmcache_info_from_pvid((const char *)&pv->id, 0))) {
+		log_error("Failed to find cached info for PV %s.", pv_dev_name(pv));
+		return -1;
+	}
+
+	ext_flags = lvmcache_ext_flags(info);
+
+	return ext_flags & PV_EXT_USED ? 1 : 0;
+}
+
 char *pv_attr_dup(struct dm_pool *mem, const struct physical_volume *pv)
 {
 	char *repstr;
+	int used = is_used_pv(pv);
+	int duplicate = lvmcache_dev_is_unchosen_duplicate(pv->dev);
 
 	if (!(repstr = dm_pool_zalloc(mem, 4))) {
 		log_error("dm_pool_alloc failed");
 		return NULL;
 	}
 
-	repstr[0] = (pv->status & ALLOCATABLE_PV) ? 'a' : '-';
+	/*
+	 * An allocatable PV is always used, so we don't need to show 'u'.
+	 */
+	if (duplicate)
+		repstr[0] = 'd';
+	else if (pv->status & ALLOCATABLE_PV)
+		repstr[0] = 'a';
+	else if (used > 0)
+		repstr[0] = 'u';
+	else
+		repstr[0] = '-';
+
 	repstr[1] = (pv->status & EXPORTED_VG) ? 'x' : '-';
 	repstr[2] = (pv->status & MISSING_PV) ? 'm' : '-';
 
@@ -228,7 +273,8 @@ uint64_t pv_mda_size(const struct physical_volume *pv)
 	return min_mda_size;
 }
 
-static int _pv_mda_free(struct metadata_area *mda, void *baton) {
+static int _pv_mda_free(struct metadata_area *mda, void *baton)
+{
 	uint64_t mda_free;
 	uint64_t *freespace = baton;
 
@@ -238,16 +284,15 @@ static int _pv_mda_free(struct metadata_area *mda, void *baton) {
 	mda_free = mda->ops->mda_free_sectors(mda);
 	if (mda_free < *freespace)
 		*freespace = mda_free;
+
 	return 1;
 }
 
-uint64_t pv_mda_free(const struct physical_volume *pv)
+uint64_t lvmcache_info_mda_free(struct lvmcache_info *info)
 {
-	struct lvmcache_info *info;
 	uint64_t freespace = UINT64_MAX;
-	const char *pvid = (const char *)&pv->id.uuid;
 
-	if ((info = lvmcache_info_from_pvid(pvid, 0)))
+	if (info)
 		lvmcache_foreach_mda(info, _pv_mda_free, &freespace);
 
 	if (freespace == UINT64_MAX)
@@ -256,6 +301,17 @@ uint64_t pv_mda_free(const struct physical_volume *pv)
 	return freespace;
 }
 
+uint64_t pv_mda_free(const struct physical_volume *pv)
+{
+	const char *pvid = (const char *)&pv->id.uuid;
+	struct lvmcache_info *info;
+
+	if ((info = lvmcache_info_from_pvid(pvid, 0)))
+		return lvmcache_info_mda_free(info);
+
+	return 0;
+}
+
 uint64_t pv_used(const struct physical_volume *pv)
 {
 	uint64_t used;
@@ -264,6 +320,7 @@ uint64_t pv_used(const struct physical_volume *pv)
 		used = 0LL;
 	else
 		used = (uint64_t) pv->pe_alloc_count * pv->pe_size;
+
 	return used;
 }
 
@@ -292,6 +349,7 @@ static int _pv_mda_set_ignored_one(struct metadata_area *mda, void *baton)
 			mda_set_ignored(vg_mda, b->mda_ignored);
 
 	mda_set_ignored(mda, b->mda_ignored);
+
 	return 1;
 }
 
@@ -344,3 +402,18 @@ unsigned pv_mda_set_ignored(const struct physical_volume *pv, unsigned mda_ignor
 	return 1;
 }
 
+struct label *pv_label(const struct physical_volume *pv)
+{
+	struct lvmcache_info *info =
+		lvmcache_info_from_pvid((const char *)&pv->id.uuid, 0);
+
+	if (info)
+		return lvmcache_get_label(info);
+
+	/* process_each_pv() may create dummy PVs that have no label */
+	if (pv->vg && pv->dev)
+		log_error(INTERNAL_ERROR "PV %s unexpectedly not in cache.",
+			  dev_name(pv->dev));
+
+	return NULL;
+}
