@@ -10,7 +10,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "dmlib.h"
@@ -28,6 +28,7 @@ struct ttree {
 	struct node *root;
 };
 
+__attribute__((nonnull(1)))
 static struct node **_lookup_single(struct node **c, unsigned int k)
 {
 	while (*c) {
@@ -86,10 +87,8 @@ int ttree_insert(struct ttree *tt, unsigned int *key, void *data)
 		count++;
 
 		while (count--) {
-			if (!(*c = _tree_node(tt->mem, k))) {
-				stack;
-				return 0;
-			}
+			if (!(*c = _tree_node(tt->mem, k)))
+				return_0;
 
 			if (count) {
 				k = *key++;
@@ -106,10 +105,8 @@ struct ttree *ttree_create(struct dm_pool *mem, unsigned int klen)
 {
 	struct ttree *tt;
 
-	if (!(tt = dm_pool_zalloc(mem, sizeof(*tt)))) {
-		stack;
-		return NULL;
-	}
+	if (!(tt = dm_pool_zalloc(mem, sizeof(*tt))))
+		return_NULL;
 
 	tt->klen = klen;
 	tt->mem = mem;
