@@ -7,9 +7,11 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
-# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-. lib/test
+SKIP_WITH_LVMPOLLD=1
+
+. lib/inittest
 
 aux prepare_vg 2
 aux apitest vglist $vg `get vg_field $vg vg_uuid | sed -e s,-,,g`
