@@ -7,9 +7,14 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
-# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-. lib/test
+SKIP_WITH_LVMLOCKD=1
+
+# disable lvmetad logging as it bogs down test systems
+export LVM_TEST_LVMETAD_DEBUG_OPTS=${LVM_TEST_LVMETAD_DEBUG_OPTS-}
+
+. lib/inittest
 
 log_name_to_count() {
 	case "$1"  in
@@ -43,7 +48,7 @@ test_lvconvert() {
 	local finish_count=$3
 	local finish_count_p1=$(($finish_count + 1))
 	local finish_log_type=$4
-	local dev_array=("$dev1" "$dev2" "$dev3" "$dev4" "$dev5")
+	local dev_array=( "$dev1" "$dev2" "$dev3" "$dev4" "$dev5" )
 	local start_log_count
 	local finish_log_count
 	local max_log_count
@@ -92,7 +97,7 @@ test_lvconvert() {
 		alloc="--alloc anywhere"
 	fi
 
-	lvconvert -m $finish_count --mirrorlog $finish_log_type \
+	lvconvert --type mirror -m $finish_count --mirrorlog $finish_log_type \
 		$vg/$lv1 $alloc
 
 	test $active || lvchange -aey $vg/$lv1
@@ -110,7 +115,6 @@ test_lvconvert() {
 	fi
 }
 
-
 aux prepare_pvs 5 5
 vgcreate -s 32k $vg $(cat DEVICES)
 
