@@ -7,12 +7,15 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
-# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 # regression test for lvmetad reporting error:
 # Internal error: LV snap_with_lvm1_meta (00000000000000000000000000000001) missing from preload metadata
 
-. lib/test
+SKIP_WITH_LVMLOCKD=1
+SKIP_WITH_LVMPOLLD=1
+
+. lib/inittest
 
 aux prepare_devs 2
 
@@ -26,3 +29,5 @@ lvcreate -s $vg/origin -n snap_with_lvm1_meta -l4
 
 # Remove volume snapper/snap_with_lvm1_meta
 lvremove -f $vg/snap_with_lvm1_meta
+
+vgremove -ff $vg
