@@ -7,9 +7,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software Foundation,
-# Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-. lib/test
+SKIP_WITH_LVMLOCKD=1
+SKIP_WITH_LVMPOLLD=1
+
+. lib/inittest
 
 aux prepare_vg 2
 
@@ -50,3 +53,5 @@ not vgchange --vgmetadatacopies 2 $vg
 # Ensure that allowed args don't cause disallowed args to get through
 #
 not vgchange -p 10 --addtag foo $vg
+
+vgremove -ff $vg
